// coding: utf-8
/*! Created by: Udo Schmal | https://www.gocher.me/ */
(function () {
  'use strict';

  var scale = ((typeof(window.devicePixelRatio) != "undefined") ? window.devicePixelRatio : 1);

  if (!Element.prototype.matches) {
    Element.prototype.matches =
      Element.prototype.msMatchesSelector ||
      Element.prototype.webkitMatchesSelector;
  }

  if (!Element.prototype.closest) {
    Element.prototype.closest = function(s) {
      var el = this;
      do {
        if (Element.prototype.matches.call(el, s)) return el;
        el = el.parentElement || el.parentNode;
      } while (el !== null && el.nodeType === 1);
      return null;
    };
  }

  function activateScript(script) {
    let clone = script.cloneNode(true);
    clone.removeAttribute("type");
    script.parentNode.insertBefore(clone, script);
    script.parentNode.removeChild(script);
  }

  // create external message
  function externalMessage(el) {
    let txt = el.dataset.external, clk;
    let external = document.createElement('div');
    external.className = 'text';
    let language = document.documentElement.lang || 'en';
    switch (language.substr(0,2)) {
      case 'de':
        txt = ('Externer Inhalt von #, der über einen Klick angeszeigt wird.').replace('#', txt);
        clk = 'Durch diesen Klick werden allerdings auch Daten an Drittplattformen übermittelt!';
        break;
      case 'fr':
        txt = ('Contenu externe de # qui peut être consulté en un clic.').replace('#', txt);
        clk = 'Cependant, ce clic transfère également les données vers des plateformes tierces!';
        break;
      case 'it':
        txt = ('Contenuti esterni da # che possono essere visualizzati con un clic.').replace('#', txt);
        clk = 'Tuttavia, questo clic trasferisce anche i dati a piattaforme di terze parti!';
        break;
      case 'es':
        txt = ('Contenido externo de # que se puede ver con un clic.').replace('#', txt);
        clk = 'Sin embargo, este clic también transfiere datos a plataformas de terceros.';
        break;
      default:
        txt = ('External content from # that can be viewed with a click.').replace('#', txt);
        clk = 'However, this click also transfers data to third-party platforms!';
        break;
    }
    external.appendChild(document.createTextNode(txt));
    external.appendChild(document.createElement('br'));
    external.appendChild(document.createTextNode(clk));
    el.appendChild(external);
    return external;
  }

  // lazy loading image
  function loadImg(el) {
    if (el.dataset.src) {
      if ((el.dataset.scale === 'true') && el.getAttribute('width') && el.getAttribute('height') && (el.dataset.src.indexOf('~') === -1)) {
        let height = parseInt(el.getAttribute('height'));
        let width = parseInt(el.getAttribute('width'));
        let dataWidth = parseInt(el.getAttribute('data-width'));
        let bounding = el.parentNode.getBoundingClientRect();
        let maxWidth = bounding.width;
        let fileName = el.dataset.src.split('.');
        let fileExt = fileName.pop();
        fileName = fileName.join('.');
        if (width > maxWidth) {
          height = Math.ceil(maxWidth * (height / width));
          width = Math.ceil(maxWidth);
          el.setAttribute('height', height);
          el.setAttribute('width', width);
        }
        if (dataWidth > width*scale) {
          el.src = fileName + '~' + Math.ceil(width * scale) + 'x' + Math.ceil(height * scale) + '.' + fileExt;
        } else {
          el.src = fileName + '~' + width + 'x' + height + '.' + fileExt;
        }
      } else {
        el.src = el.dataset.src;
      }
      el.removeAttribute('data-src');
    }
    if (el.dataset.srcset) {
      el.setAttribute('srcset', el.dataset.srcset);
      el.removeAttribute('data-srcset');
    }
    if (el.dataset.sizes) {
      el.setAttribute('sizes', el.dataset.sizes);
      el.removeAttribute('data-sizes');
    }
  }

  // lazy loading iframe
  function loadIframe(el) {
    if (el.dataset.src) {
      el.src = el.dataset.src;
      el.removeAttribute('data-src');
    }
  }

  // lazy loading video
  function loadVideo(el) {
    if (el.dataset.poster) {
      el.setAttribute('poster', el.dataset.poster);
      el.removeAttribute('data-poster');
    }
    // show in popup
    var target = el.dataset.target;
    if (target == 'popup') {
      // create popup
      function createPopup() {
        var overlay = document.createElement('div');
        overlay.className = 'overlay';
        document.body.appendChild(overlay);
        var content = document.createElement('div');
        content.className = 'popup-content-video';
        overlay.appendChild(content);
        var close = document.createElement('span');
        close.className = 'popup-close';
        overlay.appendChild(close);
        close.addEventListener('click', function (event) {
          document.body.removeChild(overlay);
        });
        return content;
      }
      // create placeholder
      let placeholder = document.createElement('div');
      placeholder.className = 'video-placeholder';
      if (el.poster) {
        placeholder.style.backgroundImage = "url('" + el.poster + "')";
      }
      el.parentNode.insertBefore(placeholder, el);
      // play button
      let play = document.createElement('div');
      play.className = 'play';
      placeholder.appendChild(play);
      function playVideo(event) {
        document.activeElement.blur();
        let dest = createPopup();
        let video = el.cloneNode(true);
        video.classList.remove('hidden');
        dest.appendChild(video);
        video.setAttribute('preload', 'auto');
        video.load();
        video.setAttribute('autoplay', 'autoplay');
        video.autoplay = true;
        video.play;
      }
      placeholder.addEventListener('click', playVideo);
    }
    // find best matching source by mediaQuery
    var source = null;
    var media = el.querySelectorAll('source[data-src]');
    if (media.length > 0) {
      for(let i = 0; i < media.length; i++) {
        if (media[i].dataset.media) {
          if (window.matchMedia(media[i].dataset.media).matches) {
            source = media[i];
            break;
          }
        } else {
          source = media[i]
        }
      }
      if (!source) {
        source = media[0];
      }
    }
    if (source && (!el.active || el.active != source)) {
      source.src = source.dataset.src;
      if (el.active) {
        el.active.removeAttribute('src');
      }
      el.active = source;
    }
    if (el.dataset.controls && el.dataset.controls == 'true') {
      el.setAttribute('controls', '');
      el.removeAttribute('data-controls');
    }
    if (target !== 'popup') {
      el.setAttribute('preload', 'auto');
      el.load();
      if (el.dataset.autoplay && el.dataset.autoplay == 'true') {
        el.removeAttribute('data-autoplay');
        el.setAttribute('autoplay', 'autoplay');
        el.autoplay = true;
        el.play();
      }
    } else {
      el.classList.add('hidden');
    }
  }

  // lazy loading external
  function loadExternal(el) {
    var src = el.dataset.src;
    if (el.dataset.external) {
      // create placeholder background image
      if (el.dataset.poster) {
        el.style.backgroundImage = "url('" + el.dataset.poster + "')";
      }/* else {
        el.style.backgroundImage = "url('/media/fallback.jpg')";
      }*/
      // external message
      var msg = externalMessage(el);

      // play button
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
      function load(event) {
        document.activeElement.blur();
        msg.classList.add('hidden');
        play.classList.add('hidden');
        var iframe = document.createElement('iframe');
        iframe.setAttribute('allowfullscreen', 'allowfullscreen');
        iframe.setAttribute('loading', 'lazy');
        iframe.src = el.dataset.src;
        el.appendChild(iframe);
        el.style.backgroundImage = 'none';
        play.removeEventListener('click', load);
      }
      play.addEventListener('click', load);
    } else {
      var iframe = document.createElement('iframe');
      iframe.setAttribute('allowfullscreen', 'allowfullscreen');
      iframe.setAttribute('loading', 'lazy');
      iframe.src = el.dataset.src;
      el.appendChild(iframe);
    }
  }

  // lazy loading background image
  function loadBackgroundImage(el) {
    if (el.dataset.backgroundimage) {
      el.style.backgroundImage = "url('" + el.dataset.backgroundimage + "')";
    }
  }

  // lazy loading youtube
  function youtube(el) {
    var player, id;
    // don't send referrer infos to external services
    if (!document.querySelector('meta[name="referrer"]')) {
      let meta = document.createElement('meta');
      meta.name = 'referrer';
      meta.content = 'no-referrer';
      document.head.appendChild(meta);
    }
    // get YouTube video id
    if (el.dataset.src && el.dataset.src.length === 11) {
      id = el.dataset.src;
    } else {
      let src = el.dataset.src;
      let regExp = /^.*((youtube\/)|(v\/)|(\/u\/\w\/)|(embed\/)|(watch\?))\??v?=?([^#&?]*).*/;
      let match = src.match(regExp);
      if (match) {
        id = (match && match[7].length == 11) ? match[7] : src;
      }
    }
    let idx = Math.floor(Math.random() * 6);
    el.id = id + '_' + idx;
    if (!el.dataset.poster) {
      el.dataset.poster = 'https://img.youtube.com/vi/' + id + '/sddefault.jpg';
    }
    // show in popup, other defined target or replace el with player
    var target = el.dataset.target || id + '_' + idx;

    if (el.dataset.autoplay) {
      load();
    }  else {
      // create placeholder background image
      if (el.dataset.poster) {
        el.style.backgroundImage = "url('" + el.dataset.poster + "')";
      }
      // show placeholder title
      if (el.title) {
        let title = document.createElement('div');
        title.className = 'title';
        title.appendChild(document.createTextNode(el.title));
        el.appendChild(title);
      }
      // show placeholder external message
      if (el.dataset.external) {
        externalMessage(el);
      }
      // show placeholder youtube like play button
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
    }
    // create popup
    function createPopup() {
      var overlay = document.createElement('div');
      overlay.className = 'overlay';
      document.body.appendChild(overlay);
      var content = document.createElement('div');
      content.className = 'popup-content';
      overlay.appendChild(content);
      var close = document.createElement('span');
      close.className = 'popup-close';
      overlay.appendChild(close);
      close.addEventListener('click', function (event) {
        player.stopVideo();
        document.body.removeChild(overlay);
      });
      return content;
    }
    // youtube player API
    function onPlayerReady(event) {
      event.target.playVideo();
    }
    function onPlayerStateChange(event) {
      if (event.data == window.YT.PlayerState.ENDED) {
        player.stopVideo();
        document.body.removeChild(overlay);
      }
    }
    // create youtube player
    function createPlayer() {
      window.YT.ready(function() {
        var dest, obj = {
          videoId: id,
          host: 'https://www.youtube-nocookie.com',
          events: {
            'onReady': onPlayerReady
          }
        };
        var sizes = [
          { width: '426', height: '240' },
          { width: '640', height: '360' },
          { width: '854', height: '480' },
          { width: '1280', height: '720' },
          { width: '1920', height: '1080' }
        ];
        if (target == 'popup') {
          let popup = createPopup();
          popup.id = 'popup-content';
          dest = 'popup-content';
          obj.events['onStateChange'] = onPlayerStateChange;
          for (var i = 0; i < sizes.length; i++) {
            if (sizes[i].width <= window.innerWidth) {
              obj.height = sizes[i].height;
              obj.width = sizes[i].width;
            } else {
              break;
            }
          }
        } else {
          dest = target;
          for (var i = 0; i < sizes.length; i++) {
            if (sizes[i].width >= el.offsetWidth) {
              obj.height = sizes[i].height;
              obj.width = sizes[i].width;
              break;
            }
          }
        }
        // create YouTube player
        player = new window.YT.Player(dest, obj);
      });
    }
    // load youtube external content
    function load() {
      // init once
      if (!document.getElementById('YouTubeApi')) {
        // load YouTube API
        var script = document.createElement('script');
        script.id = 'YouTubeApi';
        script.src = 'https://www.youtube.com/iframe_api';
        document.body.appendChild(script);
        script.addEventListener('load', createPlayer);
      } else {
        createPlayer();
      }
      document.activeElement.blur();
      if (target !== 'popup') {
        // remove placeholder click event
        el.removeEventListener('click', load);
      }
    }
    el.addEventListener('click', load);
  }

  // lazy loading vimeo
  function vimeo(el) {
    var player, id;
    // don't send referrer infos to external services
    if (!document.querySelector('meta[name="referrer"]')) {
      let meta = document.createElement('meta');
      meta.name = 'referrer';
      meta.content = 'no-referrer';
      document.head.appendChild(meta);
    }
    // get Vimeo video id
    if (el.dataset.src && !isNaN(el.dataset.src)) {
      id = el.dataset.src;
    } else {
      let src = el.dataset.src;
      let regExp = /http:\/\/(www\.)?vimeo.com\/(\d+)($|\/)/;
      let match = src.match(regExp);
      if (match){
        id = match[2];
      }
    }
    let idx = Math.floor(Math.random() * 6);
    el.id = id + '_' + idx;
    // show in popup, other defined target or replace el with player
    var target = el.dataset.target || id + '_' + idx;

    if (el.dataset.autoplay) {
      load();
    }  else {
      // create placeholder background image
      if (el.dataset.poster) {
        el.style.backgroundImage = "url('" + el.dataset.poster + "')";
      }
      // show placeholder title
      if (el.title) {
        let title = document.createElement('div');
        title.className = 'title';
        title.appendChild(document.createTextNode(el.title));
        el.appendChild(title);
      }
      // show placeholder external message
      if (el.dataset.external) {
        externalMessage(el);
      }
      // show placeholder youtube like play button
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
    }
    // create popup
    function createPopup() {
      var overlay = document.createElement('div');
      overlay.className = 'overlay';
      document.body.appendChild(overlay);
      var content = document.createElement('div');
      content.className = 'popup-content';
      overlay.appendChild(content);
      var close = document.createElement('span');
      close.className = 'popup-close';
      overlay.appendChild(close);
      close.addEventListener('click', function (event) {
        document.body.removeChild(overlay);
      });
      return content;
    }
    // load vimeo external content
    function load() {
      document.activeElement.blur();
      let iframe = document.createElement('iframe');
      iframe.src = 'https://player.vimeo.com/video/' + id + '?autoplay=1&autopause=0';
      iframe.dataset.external = 'Vimeo';
      iframe.classList.add('vimeo');
      iframe.style = el.style;
      var sizes = [
        { width: '1920', height: '1080' },
        { width: '1280', height: '720' },
        { width: '854', height: '480' },
        { width: '640', height: '360' },
        { width: '426', height: '240' }
      ];
      let dest;
      if (target === 'popup') {
        dest = createPopup();
      } else {
        dest = document.getElementById(target);
      }
      if ((el.dataset.width) && (el.dataset.height)) {
        iframe.width = el.dataset.width;
        iframe.height = el.dataset.height;
      } else {
        for (var i = 0; i < sizes.length; i++) {
          if (((target === 'popup') && (sizes[i].width <= window.innerWidth)) ||
              ((target !== 'popup') && (sizes[i].width <= dest.offsetWidth)) ||
              (i === sizes.length-1)) {
            dest.style.height = sizes[i].height + 'px';
            dest.style.width = sizes[i].width + 'px';
            iframe.height = sizes[i].height;
            iframe.width = sizes[i].width;
            break;
          }
        }
      }
      if (target === 'popup') {
        dest.appendChild(iframe);
      } else if (target === el.id) {
        el.parentNode.insertBefore(iframe, el);
        el.parentNode.removeChild(el);
      } else {
        if (dest.hasChildNodes()) {
          while (dest.firstChild) {
            dest.removeChild(dest.firstChild);
          }
        }
      }
    }
    el.addEventListener('click', load);
  }

  // lazy loading maps (OpenStreetMap)
  function loadMap(el) {
    var map;
    var marker = [];
    var border;
    const removeSpace = /\s*/g;
    const trimSpace = /^\s*|\s*$/g;
    const splitSpace = /\s+/;
    function coordinate(value) {
      value = value
        .replace(removeSpace, "")
        .split(",")
        .map(Number.parseFloat)
        .filter((num) => !Number.isNaN(num))
        .slice(0, 3);
      let temp = value[0];
      value[0] = value[1];
      value[1] = temp;
      return value;
    }
    function coordinates(value) {
      return value
        .replace(trimSpace, "")
        .split(splitSpace)
        .map(coordinate)
        .filter((coord) => {
          return coord.length >= 2;
        });
    }
    function showMap() {
      if (el.dataset.lat && el.dataset.lon) {
        if (!map) {
          // for map screenshot
          let screenshot = false;
          let zoom = el.dataset.zoom || 18;
          let minZoom = el.dataset.minzoom || 8;
          // set map tiles source
          var osm = L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
            maxZoom: 19,
            minZoom: minZoom,
            attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/" target="_blank" rel="nofollow noopener noreferrer">OpenStreetMap</a> and contributors <a href="https://creativecommons.org/licenses/by-sa/4.0/" target="_blank" rel="nofollow noopener noreferrer">CC-BY-SA</a>'
          });
          // https://josm.openstreetmap.de/ticket/23226
          /*var topo = L.tileLayer('https://tile.tracestrack.com/topo__/{z}/{x}/{y}@1x.png?key=041e2e98b93659b9bce98f956fc177e7', {
            maxZoom: 19,
            minZoom: minZoom,
            attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/" target="_blank" rel="nofollow noopener noreferrer">OpenStreetMap</a> and contributors | Map display &copy; <a href="https://www.tracestrack.com/"  rel="nofollow noopener noreferrer" target="_blank">Tracestrack</a> | <a href="https://creativecommons.org/licenses/by-sa/4.0/" target="_blank" rel="nofollow noopener noreferrer">CC-BY-SA</a>'
          });*/
          var satellite = L.tileLayer('https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}', {
            maxZoom: 19,
            minZoom: minZoom,
            attribution: 'Map data &copy; <a href="https://www.esri.com/" target="_blank" rel="nofollow noopener noreferrer">Esri</a>, i-cubed, USDA, USGS, AEX, GeoEye, Getmapping, Aerogrid, IGN, IGP, UPR-EGP, and the GIS User Community'
          });
          // initialize map
          if (screenshot) {
            map = L.map(el, { layers: osm, tap: false, scrollWheelZoom: false, zoomControl: false, attributionControl: false } ).setView([el.dataset.lat, el.dataset.lon], zoom);
          } else {
            map = L.map(el, { layers: osm, tap: false, scrollWheelZoom: false } ).setView([el.dataset.lat, el.dataset.lon], zoom);
          }
          var baseLayers;
          L.control.scale({imperial:false}).addTo(map); // remove for map screenshot
          baseLayers = {
            "OSM": osm,
            /*"Topo Map": topo,*/
            "Satellite": satellite
          };
          if (!screenshot) {
            L.control.layers(baseLayers).addTo(map);
          }
          el.style.backgroundImage = 'none';
          /*if (el.classList.contains('hitmap')) {
            map.on('map-container-resize', function () {
              map.invalidateSize(); // doesn't seem to do anything
            });
          }*/
          map.setView([el.dataset.lat, el.dataset.lon], zoom);
          // console write lat lon values
          map.on('click', function (e) {
            console.info('map click: ' + e.latlng.toString());
          });
          // console write tracking point on doubleclick
          /*map.on('click', function (e) {
            console.info('<trkpt lat="' + e.latlng.lat.toFixed(6) + '" lon="' + e.latlng.lng.toFixed(6) + '"></trkpt>');
          });*/
          // console write new center position
          /*map.on('moveend', function () {
            console.log('map center: ' + map.getCenter().toString());
          });*/
        }
        let LeafIcon = L.Icon.extend({
          options: {
            shadowUrl: 'styles/symbols/marker-shadow.png',
            iconSize:     [30, 30],
            shadowSize:   [34, 40],
            iconAnchor:   [15, 39],
            shadowAnchor: [17, 40],
            popupAnchor:  [1, -34]
          }
        });
        let LeafProtectedIcon = L.Icon.extend({
          options: {
            shadowUrl: 'styles/symbols/marker-shadow-protected.png',
            iconSize:     [30, 30],
            shadowSize:   [34, 40],
            iconAnchor:   [15, 39],
            shadowAnchor: [17, 40],
            popupAnchor:  [1, -34]
          }
        });
        var icons = {
          'done' : {
            'default' : new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-blue.png'}),
            'windmill' : new LeafIcon({iconUrl: 'styles/symbols/windmill-blades-blue.svg'}),
            'watermill': new LeafIcon({iconUrl: 'styles/symbols/watermill-weel-blue.svg'}),
            'industrialmill': new LeafIcon({iconUrl: 'styles/symbols/industrialmill-blue.svg'}),
            'windturbine': new LeafIcon({iconUrl: 'styles/symbols/windturbine-blue.svg'}),
            'goblet': new LeafIcon({iconUrl: 'styles/symbols/goblet-blue.svg'}),
            'runestone': new LeafIcon({iconUrl: 'styles/symbols/runestone-blue.svg'}),
            'gate': new LeafIcon({iconUrl: 'styles/symbols/gate-blue.svg'}),
            'wall': new LeafIcon({iconUrl: 'styles/symbols/wall-blue.svg'}),
            'fort': new LeafIcon({iconUrl: 'styles/symbols/fort-blue.svg'})
          },
          'todo' : {
            'default' : new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-red.png'}),
            'windmill' : new LeafIcon({iconUrl: 'styles/symbols/windmill-blades-red.svg'}),
            'watermill': new LeafIcon({iconUrl: 'styles/symbols/watermill-weel-red.svg'}),
            'industrialmill': new LeafIcon({iconUrl: 'styles/symbols/industrialmill-red.svg'}),
            'windturbine': new LeafIcon({iconUrl: 'styles/symbols/windturbine-red.svg'}),
            'goblet': new LeafIcon({iconUrl: 'styles/symbols/goblet-red.svg'}),
            'runestone': new LeafIcon({iconUrl: 'styles/symbols/runestone-red.svg'}),
            'gate': new LeafIcon({iconUrl: 'styles/symbols/gate-red.svg'}),
            'wall': new LeafIcon({iconUrl: 'styles/symbols/wall-red.svg'}),
            'fort': new LeafIcon({iconUrl: 'styles/symbols/fort-red.svg'})
          },
          'protected-done' : {
            'default' : new LeafProtectedIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-blue.png'}),
            'windmill' : new LeafProtectedIcon({iconUrl: 'styles/symbols/windmill-blades-blue.svg'}),
            'watermill': new LeafProtectedIcon({iconUrl: 'styles/symbols/watermill-weel-blue.svg'}),
            'industrialmill': new LeafProtectedIcon({iconUrl: 'styles/symbols/industrialmill-blue.svg'}),
            'windturbine': new LeafProtectedIcon({iconUrl: 'styles/symbols/windturbine-blue.svg'}),
            'goblet': new LeafProtectedIcon({iconUrl: 'styles/symbols/goblet-blue.svg'}),
            'runestone': new LeafProtectedIcon({iconUrl: 'styles/symbols/runestone-blue.svg'}),
            'gate': new LeafProtectedIcon({iconUrl: 'styles/symbols/gate-blue.svg'}),
            'wall': new LeafProtectedIcon({iconUrl: 'styles/symbols/wall-blue.svg'}),
            'fort': new LeafProtectedIcon({iconUrl: 'styles/symbols/fort-blue.svg'})
          },
          'protected-todo' : {
            'default' : new LeafProtectedIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-red.png'}),
            'windmill' : new LeafProtectedIcon({iconUrl: 'styles/symbols/windmill-blades-red.svg'}),
            'watermill': new LeafProtectedIcon({iconUrl: 'styles/symbols/watermill-weel-red.svg'}),
            'industrialmill': new LeafProtectedIcon({iconUrl: 'styles/symbols/industrialmill-red.svg'}),
            'windturbine': new LeafProtectedIcon({iconUrl: 'styles/symbols/windturbine-red.svg'}),
            'goblet': new LeafProtectedIcon({iconUrl: 'styles/symbols/goblet-red.svg'}),
            'runestone': new LeafProtectedIcon({iconUrl: 'styles/symbols/runestone-red.svg'}),
            'gate': new LeafProtectedIcon({iconUrl: 'styles/symbols/gate-red.svg'}),
            'wall': new LeafProtectedIcon({iconUrl: 'styles/symbols/wall-red.svg'}),
            'fort': new LeafProtectedIcon({iconUrl: 'styles/symbols/fort-red.svg'})
          }
        };
        function createMarker(el, single) {
          let marker;
          let lat = el.querySelector('.lat') ? el.querySelector('.lat').innerHTML : null;
          let lon = el.querySelector('.lon') ? el.querySelector('.lon').innerHTML : null;
          if (!lat && !lon) {
            lat = el.dataset.lat ? el.dataset.lat : null;
            lon = el.dataset.lon ? el.dataset.lon : null;
          }
          let attrPolygon = el.dataset.polygon ? el.dataset.polygon : null;
          let attrPolyline = el.dataset.polyline ? el.dataset.polyline : null;
          let attrColor = el.dataset.color ? el.dataset.color : null;
          let title = el.querySelector('h3') ? el.querySelector('h3').innerHTML : null;
          let info = el.querySelector('.address') ? el.querySelector('.address').innerHTML : null;
          let link = el.querySelector('span.links') ? el.querySelector('span.links').innerHTML : null;
          let txt = (title ? '<strong>' + title + '</strong>' : '') +
                     (info ? '<br />' + info : '') +
                     (link ? '<br />' + link : '');
          if (!single) {
            let anchor = el.id;
            if (!anchor) {
              anchor = 'c' + i;
              el.id = anchor;
            }
            txt += ', <a href="#' + anchor + '">zum Eintrag</a>';
          }
          // add marker to the map
          let status = el.classList.contains('protected') || el.parentNode.classList.contains('protected') ? 'protected-' : '';
          let work = el.classList.contains('done') ? 'done' : 'todo';
          let icon = icons[work]['default'];
          for (const [key, value] of Object.entries(icons[status + work])) {
            if (el.classList.contains(key)) {
              icon = icons[status + work][key];
              break;
            }
          }
          if (attrPolygon) {
            if (!attrColor) {
              attrColor = '#1b78c5';
            }
            let latlngs = coordinates(attrPolygon);
            // create a polygon from an array of LatLng points
            let polygon = L.polygon(latlngs, {color: attrColor, opacity: 0.5}).bindTooltip(title).addTo(map);
          } else if (attrPolyline) {
            if (!attrColor) {
              attrColor = '#1b78c5';
            }
            attrPolyline = attrPolyline.replaceAll("'", '"');
            let lineStrings = JSON.parse(attrPolyline);
            for (let u = 0; u < lineStrings.length; u++) {
              let lineString = lineStrings[u];
              let coords = lineString.coordinates;
              let notVisible = !(lineString.visible == true);
              let latlngs = coordinates(coords);
              // create a polyline from an array of LatLng points
              if (notVisible) {
                let polyline = L.polyline(latlngs, {color: attrColor, dashArray: '5, 5', dashOffset: '5'}).bindTooltip(title).addTo(map);
              } else {
                let polyline = L.polyline(latlngs, {color: attrColor}).bindTooltip(title).addTo(map);
              }
            }
          }
          if (lat && lon) {
            if (el.classList.contains('geo-item') && single) {
              // center map at this marker
              map.panTo(new L.LatLng(lat, lon));
            }
            if (!attrPolygon && !attrPolyline) {
              // add popup to the marker
              marker = L.marker([lat, lon], {icon: icon}).bindTooltip(title).addTo(map);
              marker.bindPopup(txt);
              return marker;
            }
          }
        }
        if (el.classList.contains('hitmap') || el.classList.contains('map')) {
          // remove markers from map
          let len = marker.length;
          for (var i = 0; i < len; i++) {
            map.removeLayer(marker[i]);
          }
          let hits = [];
          if (el.dataset.selector) {
            hits = document.querySelectorAll(el.dataset.selector);
          } else {
            hits = document.querySelectorAll("section.geo-item:not(.hidden)");
          }
          for (let i=0; i < hits.length; i++) {
            marker[i] = createMarker(hits[i], false);
          }
        } else if (!el.classList.contains('map')) {
          parent = el.parentNode;
          //if (parent.querySelector('.lat') && parent.querySelector('.lon')) {
            marker = createMarker(parent, true);
          //}
        }
      }
      if (el.dataset.gpx) {
        showGpx();
      }
      if (el.dataset.kml) {
        showKml();
      }
    }
    let LeafIcon = L.Icon.extend({
      options: {
        shadowUrl: 'code/OpenStreetMap/images/marker-shadow.png',
        iconSize: [25, 41],
        iconAnchor: [12, 41],
        popupAnchor: [1, -34],
        tooltipAnchor: [16, -28],
        shadowSize: [41, 41]
      }
    });
    var markerIcons = {
      'default' : new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-blue.png'}),
      'special' : new LeafIcon({iconUrl: 'code/OpenStreetMap/images/marker-icon-2x-red.png'})
    };
    function showGpx() {
      function processGpxJsonResponse(xml) {
        let els, el, i, name, desc, author, copyright, marker;
        var t, l, layers = [];
        el = xml.getElementsByTagName('name');
        if (el.length > 0) {
          name = el[0].textContent;
        }
        el = xml.getElementsByTagName('desc');
        if (el.length > 0) {
          desc = el[0].textContent;
        }
        el = xml.getElementsByTagName('author');
        if (el.length > 0) {
          author = el[0].textContent;
        }
        el = xml.getElementsByTagName('copyright');
        if (el.length > 0) {
          copyright = el[0].textContent;
        }

        // routes are <rtept> tags inside <rte> sections
        els = xml.getElementsByTagName('rte');
        for (i = 0; i < els.length; i++) {
          let route = els[i];
          el = route.getElementsByTagName('name');
          let name = el.length > 0 ? el[0].textContent : '';
          el = route.getElementsByTagName('rtept');
          let latlngs = [];
          for (let o = 0; o < el.length; o++) {
            latlngs[o] = [el[o].getAttribute('lat'), el[o].getAttribute('lon')];
          }
          // create a blue polygon from an array of LatLng points
          //let polyline = L.polyline(latlngs, {color: 'red'}).bindTooltip(name).addTo(map);
          let polygon = L.polygon(latlngs, {color: '#1b78c5', opacity: 0.5}).bindTooltip(name).addTo(map);
        }

        // tracks are <trkpt> tags in one or more <trkseg> sections in each <trk>
        els = xml.getElementsByTagName('trk');
        for (i = 0; i < els.length; i++) {
          let track = els[i];
          el = track.getElementsByTagName('name');
          let name = el.length > 0 ? el[0].textContent : '';
          el = track.getElementsByTagName('cmt');
          let notVisible = el.length > 0 ? (el[0].textContent=='not visible') : false;
          let sections = track.getElementsByTagName('trkseg');
          for (let u = 0; u < sections.length; u++) {
            let section = sections[u];
            let latlngs = [];
            let el = section.getElementsByTagName('trkpt');
            for (let o = 0; o < el.length; o++) {
              latlngs[o] = [el[o].getAttribute('lat'), el[o].getAttribute('lon')];
            }
            // create a blue polyline from an array of LatLng points
            if (notVisible) {
              let polyline = L.polyline(latlngs, {color: '#1b78c5', dashArray: '5, 5', dashOffset: '5'}).bindTooltip(name).addTo(map);
            } else {
              let polyline = L.polyline(latlngs, {color: '#1b78c5'}).bindTooltip(name).addTo(map);
            }
          }
        }

        // parse waypoints and add markers for each of them
        els = xml.getElementsByTagName('wpt');
        for (i = 0; i < els.length; i++) {
          let _lat = els[i].getAttribute('lat');
          let _lon =  els[i].getAttribute('lon');
          el = els[i].getElementsByTagName('name');
          let name = el.length > 0 ? el[0].textContent : '';
          el = els[i].getElementsByTagName('desc');
          let desc = el.length > 0 ? el[0].textContent : '';
          el = els[i].getElementsByTagName('sym');
          let sym = el.length > 0 ? el[0].textContent : null;
          el = els[i].getElementsByTagName('type');
          let type = el.length > 0 ? typeEl[0].textContent : null;
          // add waypoint marker based on the waypoint symbol key.
          let symIcon;
          if (sym && markerIcons[sym]) {
            symIcon = markerIcons[sym];
          } else if (type && markerIcons[type]) {
            symIcon = markerIcons[type];
          } else {
            symIcon = markerIcons['default'];
          }
          marker = L.marker([_lat, _lon], {title: name, icon: symIcon, type: 'waypoint'}).addTo(map);
          marker.bindPopup("<b>" + name + "</b>" + (desc.length > 0 ? '<br>' + desc : ''));
        }
      }
      function sendGpxJsonRequest(url) {
        var request = new XMLHttpRequest();
        request.open('GET', url, true);
        request.onload = function() {
          if (this.status >= 200 && this.status < 400) {
            var data = this.responseXML;
            processGpxJsonResponse(data);
          }
        };
        request.overrideMimeType('text/xml');
        request.send();
      }
      if (el.dataset.gpx) {
        sendGpxJsonRequest(el.dataset.gpx);
      }
    }
    function showKml() {
      function processKmlJsonResponse(xml) {
        let els, el, i, marker;
        var t, l, layers = [];
        // parse all placemarks
        els = xml.getElementsByTagName('Placemark');
        for (i = 0; i < els.length; i++) {
          let placemark = els[i];
          el = placemark.getElementsByTagName('name');
          let name = el.length > 0 ? el[0].textContent : '';
          el = placemark.getElementsByTagName('description');
          let description = el.length > 0 ? el[0].textContent : '';
          el = placemark.getElementsByTagName('styleUrl');
          let style = el.length > 0 ? el[0].textContent.substring(1) : '';
          let notVisible = style.includes('invisible');
          // parse points and add markers for each of them
          let points = placemark.getElementsByTagName('Point');
          for (let u = 0; u < points.length; u++) {
            let symIcon;
            if (style && markerIcons[style]) {
              symIcon = markerIcons[style];
            } else {
              symIcon = markerIcons['default'];
            }
            let coords = points[u].getElementsByTagName('coordinates');
            let latlngs = coordinate(coords[0].textContent);
            marker = L.marker(latlngs, {icon: symIcon, type: 'waypoint'}).bindTooltip(name).addTo(map);
            marker.bindPopup("<b>" + name + "</b>" + (description.length > 0 ? '<br>' + description : ''));
          }
          // parse line strings and add polylines for each of them
          let lineStrings = placemark.getElementsByTagName('LineString');
          for (let u = 0; u < lineStrings.length; u++) {
            let lineString = lineStrings[u];
            let coords = lineString.getElementsByTagName('coordinates');
            let latlngs = coordinates(coords[0].textContent);
            // create a blue polyline from an array of LatLng points
            if (notVisible) {
              let polyline = L.polyline(latlngs, {color: '#1b78c5', dashArray: '5, 5', dashOffset: '5'}).bindTooltip(name).addTo(map);
            } else {
              let polyline = L.polyline(latlngs, {color: '#1b78c5'}).bindTooltip(name).addTo(map);
            }
          }
          // parse linear rings and add polygons for each of them
          let linearRings = placemark.getElementsByTagName('LinearRing');
          for (let u = 0; u < linearRings.length; u++) {
            let linearRing = linearRings[u];
            let coords = linearRing.getElementsByTagName('coordinates');
            let latlngs = coordinates(coords[0].textContent);
            // create a blue polygon from an array of LatLng points
            let polygon = L.polygon(latlngs, {color: '#1b78c5', opacity: 0.5}).bindTooltip(name).addTo(map);
          }
        }
      }
      function sendKmlJsonRequest(url) {
        var request = new XMLHttpRequest();
        request.open('GET', url, true);
        request.onload = function() {
          if (this.status >= 200 && this.status < 400) {
            var data = this.responseXML;
            processKmlJsonResponse(data);
          }
        };
        request.overrideMimeType('text/xml');
        request.send();
      }
      if (el.dataset.kml) {
        sendKmlJsonRequest(el.dataset.kml);
      }
    }

    function showRegion(region) {
      function processGeoJsonResponse(data) {
        data.features.forEach(function(feature) {
          border.addData(feature);
        });
        var bbox = data.features[0].bbox;
        var topLeft = L.latLng(bbox[1], bbox[0]);
        var bottomRight = L.latLng(bbox[3], bbox[2]);
        var countyBounds = L.latLngBounds(topLeft, bottomRight);
        map.flyToBounds(countyBounds);
      }
      function sendGeoJsonRequest(type, code, geojson) {
        var url = '';
        if (geojson) {
          url = geojson;
        } else {
          url = 'https://nominatim.openstreetmap.org/search' +
            '?polygon_geojson=1' +
            '&format=geojson' +
            '&polygon_threshold=0.0' +
            '&' + type + '=' + code;
        }
        var request = new XMLHttpRequest();
        request.open('GET', url, false);
        request.onload = function() {
          if (this.status >= 200 && this.status < 400) {
            var data = JSON.parse(this.response);
            processGeoJsonResponse(data);
          }
        };
        request.send();
      }
      if (border) {
        map.removeLayer(border);
      }
      if (region) {
        border = L.geoJSON();
        map.addLayer(border);
        let el = document.querySelector('div.geo-region[data-region="' + region + '"]');
        let geojson = null;
        let code = '';
        let type = '';
        if (el) {
          geojson = el.getAttribute('data-geojson');
          code = el.getAttribute('data-info');
          type = el.getAttribute('data-type');
        }
        if (type == '') {
          type = 'county';
        }
        if (code == '') {
          code = region;
        }
        sendGeoJsonRequest(type, code, geojson);
      }
    }
    // create placeholder background image
    if (el.dataset.poster) {
      el.style.backgroundImage = "url('" + el.dataset.poster + "')";
    }
    let external = document.createElement('div');
    external.className = 'external-text';
    let language = document.documentElement.lang || 'en';
    let copyright = document.createElement('div');
    copyright.className = 'copyright';
    copyright.innerHTML = 'Map data &copy; <a href="https://www.openstreetmap.org/" target="_blank" rel="nofollow noopener noreferrer">OpenStreetMap</a> and contributors <a href="https://creativecommons.org/licenses/by-sa/4.0/" target="_blank" rel="nofollow noopener noreferrer">CC-BY-SA</a>';
    el.appendChild(copyright);
    let txt;
    switch (language.substr(0,2)) {
      case 'de': txt = 'Lädt externe Inhalte von OpenStreetMap und Mitwirkenden'; break;
      case 'fr': txt = 'Charger du contenu externe depuis OpenStreetMap et les contributeurs'; break;
      case 'it': txt = 'Carica contenuto esterno da OpenStreetMap e collaboratori'; break;
      case 'es': txt = 'Cargar contenido externo de OpenStreetMap y colaboradores'; break;
      default: txt = 'Load external Content from OpenStreetMap and conributors'; break;
    }
    external.appendChild(document.createTextNode(txt));
    el.appendChild(external);
    if (!el.classList.contains('hitmap')) {
      // play button
      var play = document.createElement('div');
      play.className = 'external-play';
      el.appendChild(play);
    } else {
      // filter by legend
      let legend = document.querySelector('fieldset.geo-legend');
      if (legend) {
        function filter() {
          // init show all
          let items = document.querySelectorAll("section.geo-item");
          for (let i=0; i < items.length; i++) {
            items[i].classList.remove('hidden');
          }
          // get checked classes
          let checked = legend.querySelectorAll('input:checked');
          let arr = [];
          for (let i=0; i < checked.length; i++) {
            arr.push(checked[i].name);
          }
          // hide if item does not containes checked class
          for (let i=0; i < items.length; i++) {
            if (!arr.some(className => items[i].classList.contains(className))) {
              items[i].classList.add('hidden');
            }
          }
          // hide if item does not contains search value
          let input = legend.querySelector('input');
          let search = input.value;
          let words = search.toLowerCase().split(' ');
          for (let i=0; i < items.length; i++) {
            if (!words.every(word => items[i].innerText.toLowerCase().search(word) !== -1)) {
              items[i].classList.add('hidden');
            }
          }
          // hide if item is not in selected region
          let regions = document.querySelectorAll('div.geo-region');
          let select = legend.querySelector('select');
          let selection = select.value;
          for (let i=0; i < regions.length; i++) {
            if ((selection !== '') && (regions[i].getAttribute('data-region') != selection)) {
              let items = regions[i].querySelectorAll("section.geo-item");
              for (let o=0; o < items.length; o++) {
                items[o].classList.add('hidden');
              }
            }
            let notHidden = regions[i].querySelectorAll("section.geo-item:not(.hidden)");
            if (notHidden && notHidden.length > 0) {
              regions[i].classList.remove('hidden');
            } else {
              regions[i].classList.add('hidden');
            }
          }
          // result count
          let all = 0;
          let done = 0;
          let hits = document.querySelectorAll("section.geo-item:not(.hidden)");
          for (let i=0; i < hits.length; i++) {
            all++;
            if (hits[i].classList.contains('done')) {
              done++;
            }
          }
          let el_all = legend.querySelector('span.all');
          if (el_all) {
            el_all.innerHTML = all;
          }
          let el_done =legend.querySelector('span.done');
          if (el_done) {
            el_done.innerHTML = done;
          }
          // update map
          showMap();
          showRegion(selection);
        }
        if (!legend.querySelector('input[type=checkbox]')) {
          if ("geolocation" in navigator) {
            // geolocation support
            let loc = document.createElement('span');
            legend.insertBefore(loc, legend.firstChild);
            let btn = document.createElement('input');
            btn.id = 'geo_location';
            btn.type = 'image';
            btn.src = '/styles/symbols/geo-location.svg';
            btn.title = 'set geolocation';
            legend.insertBefore(btn, legend.firstChild);
            var mapcenter = null;
            let LeafIcon = L.Icon.extend({
              options: {
                iconSize:     [30, 30],
                iconAnchor:   [15, 15]
              }
            });
            var geoLocationIcon = new LeafIcon({iconUrl: 'styles/symbols/geo-location.svg'});
            var actPosition;
            function setGeoPosition(position) {
              loc.innerHTML = "Last Sync: " + new Date().toLocaleTimeString('de-DE');
              if (!actPosition ||
                  (position.coords.latitude !== actPosition.coords.latitude) ||
                  (position.coords.longitude !== actPosition.coords.longitude)) {
                // center map at this marker
                if (!actPosition) {
                  map.panTo(new L.LatLng(position.coords.latitude, position.coords.longitude));
                }
                actPosition = position;
                // remove center marker from map
                if (mapcenter) {
                  map.removeLayer(mapcenter);
                }
                // set center marker
                mapcenter = L.marker([position.coords.latitude, position.coords.longitude], {icon: geoLocationIcon}).addTo(map);
              }
            }
            let geoWatcher = null;
            function errPosition (error) {
              //console.error(`ERROR(${error.code}): ${error.message}`);
              switch(error.code) {
                case error.PERMISSION_DENIED:
                  loc.innerHTML = "User denied the request for Geolocation.";
                  break;
                case error.POSITION_UNAVAILABLE:
                  loc.innerHTML = "Location information is unavailable.";
                  break;
                /*case error.TIMEOUT:
                  loc.innerHTML = "The request to get user location timed out.";
                  break;*/
                case error.UNKNOWN_ERROR:
                  loc.innerHTML = "An unknown error occurred.";
                  break;
              }
            }
            function getCurrentGeoLocation (event) {
              if (!geoWatcher) {
                if (!map) {
                  filter();
                }
                navigator.geolocation.getCurrentPosition(setGeoPosition, errPosition);
                let options = {
                  enableHighAccuracy: true,
                  timeout: 5000,
                  maximumAge: 0,
                };
                geoWatcher = navigator.geolocation.watchPosition(setGeoPosition, errPosition, options);
              } else {
                navigator.geolocation.clearWatch(geoWatcher);
                geoWatcher = null;
                loc.innerHTML = '';
                // remove center marker from map
                if (mapcenter) {
                  map.removeLayer(mapcenter);
                  actPosition = null;
                }
              }
            }
            btn.addEventListener('click', getCurrentGeoLocation);
          }
          // add class checkboxes
          let labels = legend.querySelectorAll('label[class]');
          for (let i=0; i < labels.length; i++) {
            let checkbox = document.createElement('input');
            checkbox.type = 'checkbox';
            checkbox.name = labels[i].getAttribute('data-for');
            checkbox.checked = true;
            checkbox.addEventListener('change', filter);
            labels[i].parentNode.insertBefore(checkbox, labels[i]);
            labels[i].setAttribute('for', labels[i].getAttribute('data-for'));
          }
          // create region select
          let select = document.createElement('select');
          select.name = 'region';
          select.addEventListener('change', filter);
          let all = document.createElement('option');
          all.value = '';
          all.appendChild(document.createTextNode('alle'));
          select.appendChild(all);
          let regions = document.querySelectorAll('div.geo-region');
          for (let i=0; i < regions.length; i++) {
            let h = regions[i].querySelector('h2');
            if (!h) {
              h = regions[i].querySelector('h3');
            }
            if (h) {
              regions[i].setAttribute('data-region', h.innerHTML);
              let option = select.querySelector('option[value="' + h.innerHTML + '"]');
              if (!option) {
                option = document.createElement('option');
                option.value = h.innerHTML;
                option.appendChild(document.createTextNode(h.innerHTML));
                select.appendChild(option);
              }
            }
          }
          legend.insertBefore(select, legend.firstChild);
          let label = document.createElement('label');
          label.setAttribute('for', 'region');
          label.appendChild(document.createTextNode('Region'));
          legend.insertBefore(label, select);
          // crerate search field
          let br = document.createElement('br');
          legend.insertBefore(br, legend.firstChild);
          let input = document.createElement('input');
          input.type = 'search';
          input.name = 'search';
          input.addEventListener('keyup', filter);
          legend.insertBefore(input, legend.firstChild);
          label = document.createElement('label');
          label.setAttribute('for', 'searchquery');
          label.appendChild(document.createTextNode('Suche'));
          legend.insertBefore(label, input);
        }
      }
    }
    function loadMap(event) {
      external.parentNode.removeChild(external);
      copyright.parentNode.removeChild(copyright);
      if (play) {
        play.parentNode.removeChild(play);
      }
      showMap(el);
      el.removeEventListener('click', loadMap);
    }
    el.addEventListener('click', loadMap);
  }

  // lazy loading twitter
  function twitter(el) {
    if (el.dataset.autoload) {
      load();
    }  else {
      // create placeholder background image
      el.style.backgroundImage = "url('/media/twitter-background.png')";
      // show placeholder external message
      if (el.dataset.external) {
        var msg = externalMessage(el);
      }
      // show placeholder twitter like bird
      var play = document.createElement('div');
      play.className = 'play';
      el.appendChild(play);
    }
    function load() {
      let script = document.querySelector('script[src="https://platform.twitter.com/widgets.js"][type="text/plain"]');
      if (script) {
        activateScript(script);
      }
      // remove placeholder click event
      el.removeEventListener('click', load);
      if (msg) {
        msg.classList.add('hidden');
      }
      if (play) {
        play.classList.add('hidden');
      }
    }
    el.addEventListener('click', load);
  }

  // viewport intersection
  function Intersection(els, callback) {
    // create intersection observer
    let lazyobserver = new IntersectionObserver(function(entries, observer) {
      for (var i=0; i < entries.length; i++) {
        if (entries[i].isIntersecting) {
          callback(entries[i].target);
          //if (entries[i].target.tagName.toLowerCase() !== 'video') {
            lazyobserver.unobserve(entries[i].target);
          //}
        }
      }
    }, {});
    // start observing
    for (var i=0; i < els.length; i++) {
      lazyobserver.observe(els[i]);
    }
  }

  // observe elements that used to be lazy loaded
  function observeElements(dom) {
    // activate viewport for image loading
    var images = dom.querySelectorAll("img[data-src]");
    for (var i=0; i < images.length; i++) {
      // don't load images lazy, that are in viewport
      let rect = images[i].getBoundingClientRect();
      if (rect.top <= window.innerHeight && rect.top >= -rect.height &&
          rect.left <= window.innerWidth && rect.left >= -rect.width &&
          !!(images[i].offsetWidth || images[i].offsetHeight || images[i].getClientRects().length)) {
        images[i].removeAttribute('loading');
        loadImg(images[i]);
      } else if (!images[i].getAttribute('loading')) {
        images[i].setAttribute('loading', 'lazy')
      }
    }
    images = dom.querySelectorAll("img[loading='lazy']");
    if (images.length > 0) {
      // load all images before print
      function beforeprint(event) {
        if ("loading" in HTMLImageElement.prototype) {
          for (var i=0; i < images.length; i++) {
            images[i].setAttribute('loading', 'eager');
          }
        } else {
          for (var i=0; i < images.length; i++) {
            loadImg(images[i]);
          }
        }
        function sleep(milliseconds) {
        const date = Date.now();
          let currentDate = null;
          do {
            currentDate = Date.now();
          } while (currentDate - date < milliseconds);
        }
        sleep(1000);
      }
      window.addEventListener("beforeprint", beforeprint);
      if (window.matchMedia) {
        var mediaQueryList = window.matchMedia('print');
        mediaQueryList.addListener(function (mql) {
          if (mql.matches) {
            beforeprint();
          }
        });
      }

      // use native lazy loading
      if ("loading" in HTMLImageElement.prototype) {
        // move attribute value from data-src to src
        console.info('viewport: Image lazy loading native');
        for (var i=0; i < images.length; i++) {
          loadImg(images[i]);
        }
      } else {
        // use IntersectionObserver
        console.info('viewport: Image lazy loading by IntersectionObserver');
        new Intersection(images, loadImg);
      }
    }

    // activate viewport for iframe loading
    var iframes = dom.querySelectorAll("iframe[data-src]");
    for (var i=0; i < iframes.length; i++) {
      if (!iframes[i].getAttribute('loading')) {
        iframes[i].setAttribute('loading', 'lazy')
      }
    }
    iframes = dom.querySelectorAll("iframe[loading='lazy']");
    if (iframes.length > 0) {
      if ('loading' in HTMLIFrameElement.prototype) {
        // native support, move attribute value from data-src to src
        console.info('viewport: IFrame lazy loading native');
        for (var i=0; i < iframes.length; i++) {
          loadIframe(iframes[i]);
        }
      } else {
        // use IntersectionObserver
        console.info('viewport: IFrame lazy loading by IntersectionObserver');
        new Intersection(iframes, loadIframe);
      }
    }

    // activate viewport for video loading
    var videos = dom.querySelectorAll("video[data-loading='lazy']");
    if (videos.length > 0) {
      console.info('viewport: Video lazy loading by IntersectionObserver');
      new Intersection(videos, loadVideo);
    }

    // activate viewport for youtube loading
    var media = dom.querySelectorAll('.youtube[data-src]');
    if (media.length > 0) {
      console.info('viewport: YouTube lazy loading by IntersectionObserver');
      new Intersection(media, youtube);
    }

    // activate viewport for vimeo loading
    var media = dom.querySelectorAll('.vimeo[data-src]');
    if (media.length > 0) {
      console.info('viewport: Vimeo lazy loading by IntersectionObserver');
      new Intersection(media, vimeo);
    }

    // activate viewport for external content
    var external = dom.querySelectorAll('div.external[data-src]');
    if (external.length > 0) {
      console.info('viewport: External lazy loading by IntersectionObserver');
      new Intersection(external, loadExternal);
    }

    // activate viewport for twitter loading
    var twttr = document.querySelectorAll('div.twitter');
    if (twttr.length > 0) {
      console.info('viewport: Twitter lazy loading by IntersectionObserver');
      new Intersection(twttr, twitter);
    }

    // activate viewport for background images
    var background = dom.querySelectorAll('[data-backgroundimage]');
    if (background.length > 0) {
      console.info('viewport: Background image lazy loading by IntersectionObserver');
      new Intersection(background, loadBackgroundImage);
    }

    // activate viewport for osm and geo-maps (OpenStreetMap)
    var maps = dom.querySelectorAll('.osm');
    if (maps.length > 0) {
      if (!document.getElementById('leaflet.css')) {
        var style = document.createElement('link');
        style.id = 'leaflet.css';
        style.type = 'text/css';
        style.rel = 'stylesheet';
        style.href = '/code/OpenStreetMap/leaflet.css';
        document.head.appendChild(style);
      }
      if (!document.getElementById('leaflet.js')) {
        var script = document.createElement('script');
        script.id = 'leaflet.js';
        script.src = '/code/OpenStreetMap/leaflet.min.js';
        script.addEventListener('load', initMaps);
        document.body.appendChild(script);
      } else {
        initMaps();
      }
      function initMaps() {
        console.info('viewport: Map lazy loading by IntersectionObserver');
        if (maps.length > 0) {
          new Intersection(maps, loadMap);
        }
      }
      // load all maps before print
      function beforeprintMap(event) {
        for (var i=0; i < maps.length; i++) {
          loadMap(maps[i])
        }
        function sleep(milliseconds) {
        const date = Date.now();
          let currentDate = null;
          do {
            currentDate = Date.now();
          } while (currentDate - date < milliseconds);
        }
        sleep(1000);
      }
      window.addEventListener("beforeprint", beforeprintMap);
      if (window.matchMedia) {
        var mediaQueryList = window.matchMedia('print');
        mediaQueryList.addListener(function (mql) {
          if (mql.matches) {
            beforeprintMap();
          }
        });
      }
    }
    // activate print for lazy loading poster images
    var poster = dom.querySelectorAll('[data-poster]');
    if (poster.length > 0) {
      // load all poster images before print
      function beforeprintPoster(event) {
        for (var i=0; i < poster.length; i++) {
          // create placeholder background image
          if (poster[i].dataset.poster) {
            poster[i].style.backgroundImage = "url('" + poster[i].dataset.poster + "')";
          }
        }
        function sleep(milliseconds) {
        const date = Date.now();
          let currentDate = null;
          do {
            currentDate = Date.now();
          } while (currentDate - date < milliseconds);
        }
        sleep(1000);
      }
      window.addEventListener("beforeprint", beforeprintPoster);
      if (window.matchMedia) {
        var mediaQueryList = window.matchMedia('print');
        mediaQueryList.addListener(function (mql) {
          if (mql.matches) {
            beforeprintPoster();
          }
        });
      }
    }

  }
  observeElements(document.body);

  // dom mutations
  function Mutation(els) {
    let mutationObserver = new MutationObserver(function(mutations, observer) {
      for (let o=0; o < mutations.length; o++) {
        let mutation = mutations[o];
        for (let i=0; i < mutation.addedNodes.length; i++) {
          let node = mutation.addedNodes[i];
          if (!node.tagName) {
            continue; // not an element
          }
          observeElements(node);
        }
      }
    });
    // start observing
    for (var i=0; i < els.length; i++) {
      mutationObserver.observe(els[i], { childList: true, subtree: true });
    }
  }
  var mutations = document.querySelectorAll("[data-mutation='true']");
  if (mutations.length > 0) {
    new Mutation(mutations);
  }

  // expose obeserve method globally
  window.observeElements = observeElements;
})();
